@@ -118,6 +118,8 @@ class GridStateFromFile(GridValue):
         # for the two following vector, the convention is the following: False(line is disconnected) / True(line is connected)
         self.hazards = None   # numpy array representing the outage (unplanned), same size as the number of powerlines on the _grid.
         self.maintenance = None  # numpy array representing the _maintenance (planned withdrawal of a powerline), same size as the number of powerlines on the _grid.
+        self.maintenance_time = None
+        self.maintenance_duration = None
 
         self.current_index = -1
         self.sep = sep
@@ -418,13 +420,11 @@ class GridStateFromFile(GridValue):
         self._order_hazards = np.argsort(order_backend_hazards)
         self._order_maintenance = np.argsort(order_backend_maintenance)
 
-        self._init_attrs(load_p, load_q, prod_p, prod_v, hazards=hazards, maintenance=maintenance)
-
-        self.curr_iter = 0
-        if self.maintenance is not None:
-            n_ = self.maintenance.shape[0]
-        elif self.hazards is not None:
-            n_ = self.hazards.shape[0]
+        # retrieve total number of rows
+        if maintenance is not None:
+            n_ = maintenance.shape[0]
+        elif hazards is not None:
+            n_ = hazards.shape[0]
         else:
             n_ = None
             for fn in ["prod_p", "load_p", "prod_v", "load_q"]:
@@ -445,6 +445,10 @@ class GridStateFromFile(GridValue):
             # from these data.
             self.max_iter = self.n_ -1
 
+        self._init_attrs(load_p, load_q, prod_p, prod_v, hazards=hazards, maintenance=maintenance)
+
+        self.curr_iter = 0
+
     @staticmethod
     def _file_len(fname, ext_):
         # i = -1
