@@ -1,10 +1,11 @@
 __all__ = [
     "ChronicsHandler",
-    "ChangeNothing",
     "GridValue",
+    "ChangeNothing",
+    "MultiFolder",
     "GridStateFromFile",
     "GridStateFromFileWithForecasts",
-    "MultiFolder",
+    "GridStateFromFileWithForecastsWithMaintenance",
     "ReadPypowNetData"
 ]
 
@@ -15,3 +16,4 @@ from grid2op.Chronics.GridStateFromFile import GridStateFromFile
 from grid2op.Chronics.GridStateFromFileWithForecasts import GridStateFromFileWithForecasts
 from grid2op.Chronics.MultiFolder import Multifolder
 from grid2op.Chronics.ReadPypowNetData import ReadPypowNetData
+from grid2op.Chronics.GSFFWFWM import GridStateFromFileWithForecastsWithMaintenance
