@@ -12,8 +12,9 @@ from abc import ABC, abstractmethod
 
 from grid2op.Observation import BaseObservation
 from grid2op.Exceptions import PlotError
-
 from grid2op.PlotGrid.LayoutUtil import layout_obs_sub_load_and_gen
+from grid2op.PlotGrid.PlotUtil import PlotUtil as pltu
+
 
 class BasePlot(ABC):
     """
@@ -38,11 +39,15 @@ class BasePlot(ABC):
                  observation_space,
                  width=800,
                  height=600,
-                 grid_layout=None):
+                 scale=2000.0,
+                 grid_layout=None,
+                 parallel_spacing=3.0):
 
         self.observation_space = observation_space
         self.width = width
         self.height = height
+        self.scale = scale
+        self._parallel_spacing = parallel_spacing
 
         self._info_to_units = {
             "rho": "%",
@@ -50,18 +55,17 @@ class BasePlot(ABC):
             "p": "MW",
             "v":"kV"
         }
-        self._lines_info = ["rho", "a", "p", "v"]
-        self._loads_info = ["p", "v"]
-        self._gens_info = ["p", "v"]
+        self._lines_info = ["rho", "a", "p", "v", None]
+        self._loads_info = ["p", "v", None]
+        self._gens_info = ["p", "v", None]
 
         self._grid_layout = self.compute_grid_layout(observation_space, grid_layout)
-        self._layout_keys = list(self._grid_layout.keys())
 
         # Augment observation_space with dummy observation data
         # so we can use it as an observation for plotting just the layout or custom infos
         self.observation_space.topo_vect = np.ones(self.observation_space.dim_topo, dtype=np.int)
         self.observation_space.line_status = np.full(self.observation_space.n_line, True)
-        self.observation_space.rho = np.full(self.observation_space.n_line, 0.1)
+        self.observation_space.rho = np.full(self.observation_space.n_line, 0.0)
         self.observation_space.p_or = np.ones(self.observation_space.n_line)
 
     @abstractmethod
@@ -95,40 +99,46 @@ class BasePlot(ABC):
 
         This should return a native python ``dict`` 
         in the same format as observation_space.grid_layout :
-        ```
+
+        .. code-block:: python
+
             {
               "substation1_name": [x_coord, y_coord],
               "substation2_name": [x_coord, y_coord],
-               ...
+              [...],
               "load1_name": [x_coord, y_coord],
-              ...
+              [...], 
               "gen1_name": [x_coord, y_coord],
-              ...
+              [...]
             }
-        ```
+        
         Note that is must contain at least the positions for the substations.
         The loads and generators will be skipped if missing. 
 
-        By default, if :grid_layout: is provided this is returned, 
+        By default, if `grid_layout` is provided this is returned, 
         otherwise returns observation_space.grid_layout
+
         Parameters
         ----------
-        observation_space: ``grid2op.Observation.ObservationSpace`
+
+        observation_space: ``grid2op.Observation.ObservationSpace``
+             The observation space of the environment
 
         grid_layout: ``dict`` or ``None``
+             A dictionary containing the coordinates for each substation.
         """
         # We need an intial layout to work with
         use_grid_layout = None
         if grid_layout != None:
             use_grid_layout = grid_layout
-        else:
+        elif observation_space.grid_layout is not None:
             use_grid_layout = observation_space.grid_layout
-        if use_grid_layout is None:
+        else:
             raise PlotError("No grid layout provided for plotting")
 
         # Compute loads and gens positions using a default implementation
-        observation_space.grid_layout = updated_grid_layout
-        return layout_obs_sub_load_and_gen(observation_space, use_initial=True)
+        observation_space.grid_layout = use_grid_layout
+        return layout_obs_sub_load_and_gen(observation_space, scale=self.scale, use_initial=True)
 
     @abstractmethod
     def draw_substation(self, figure, observation,
@@ -347,7 +357,7 @@ class BasePlot(ABC):
             draw_fn = self.update_substation
 
         for sub_idx in range(observation.n_sub):
-            sub_name = self._layout_keys[sub_idx]
+            sub_name = observation.name_sub[sub_idx]
             sub_x = self._grid_layout[sub_name][0]
             sub_y = self._grid_layout[sub_name][1]
             draw_fn(figure, observation,
@@ -364,11 +374,13 @@ class BasePlot(ABC):
         for load_idx, load_name in enumerate(observation.name_load):
             if load_name not in self._grid_layout:
                 continue
-            load_value = np.round(load_values[load_idx], 2)
+            load_value = None
+            if load_values is not None:
+                load_value = np.round(float(load_values[load_idx]), 2)
             load_x = self._grid_layout[load_name][0]
             load_y = self._grid_layout[load_name][1]
             load_subid = observation.load_to_subid[load_idx]
-            load_subname = self._layout_keys[load_subid]
+            load_subname = observation.name_sub[load_subid]
             load_bus = topo[topo_pos[load_idx]]
             load_bus = load_bus if load_bus > 0 else 0
             sub_x = self._grid_layout[load_subname][0]
@@ -388,11 +400,13 @@ class BasePlot(ABC):
         for gen_idx, gen_name in enumerate(observation.name_gen):
             if gen_name not in self._grid_layout:
                 continue
-            gen_value = np.round(gen_values[gen_idx], 2)
+            gen_value = None
+            if gen_values is not None:
+                gen_value = np.round(float(gen_values[gen_idx]), 2)
             gen_x = self._grid_layout[gen_name][0]
             gen_y = self._grid_layout[gen_name][1]
             gen_subid = observation.gen_to_subid[gen_idx]
-            gen_subname = self._layout_keys[gen_subid]
+            gen_subname = observation.name_sub[gen_subid]
             gen_bus = topo[topo_pos[gen_idx]]
             gen_bus = gen_bus if gen_bus > 0 else 0
             sub_x = self._grid_layout[gen_subname][0]
@@ -413,21 +427,42 @@ class BasePlot(ABC):
 
         for line_idx in range(observation.n_line):
             line_or_sub = observation.line_or_to_subid[line_idx]
-            line_or_sub_name = self._layout_keys[line_or_sub]
+            line_or_sub_name = observation.name_sub[line_or_sub]
             line_ex_sub = observation.line_ex_to_subid[line_idx]
-            line_ex_sub_name = self._layout_keys[line_ex_sub]
+            line_ex_sub_name = observation.name_sub[line_ex_sub]
             line_name = "line_{}_{}".format(line_or_sub, line_ex_sub)
             line_status = True
             line_status = observation.line_status[line_idx]
-            line_value = np.round(line_values[line_idx], 2)
+            line_value = None
+            if line_values is not None:
+                line_value = np.round(float(line_values[line_idx]), 2)
+                
             line_or_bus = topo[line_or_pos[line_idx]]
             line_or_bus = line_or_bus if line_or_bus > 0 else 0
             line_or_x = self._grid_layout[line_or_sub_name][0]
             line_or_y = self._grid_layout[line_or_sub_name][1]
             line_ex_bus = topo[line_ex_pos[line_idx]]
-            line_ex_bus = line_or_bus if line_or_bus > 0 else 0
+            line_ex_bus = line_ex_bus if line_ex_bus > 0 else 0
             line_ex_x = self._grid_layout[line_ex_sub_name][0]
             line_ex_y = self._grid_layout[line_ex_sub_name][1]
+
+            # Special case for parralel lines
+            tmp = self.observation_space.get_lines_id(from_=line_or_sub,
+                                                      to_=line_ex_sub)
+            if len(tmp) > 1:
+                ox, oy = pltu.orth_norm_from_points(line_or_x, line_or_y,
+                                                    line_ex_x, line_ex_y)
+                if line_idx == tmp[0]:
+                    line_or_x += ox * self._parallel_spacing
+                    line_or_y += oy * self._parallel_spacing
+                    line_ex_x += ox * self._parallel_spacing
+                    line_ex_y += oy * self._parallel_spacing
+                else:
+                    line_or_x -= ox * self._parallel_spacing
+                    line_or_y -= oy * self._parallel_spacing
+                    line_ex_x -= ox * self._parallel_spacing
+                    line_ex_y -= oy * self._parallel_spacing
+
             draw_fn(figure, observation,
                     line_idx, line_name, line_status,
                     line_value, line_unit,
@@ -494,19 +529,22 @@ class BasePlot(ABC):
         if line_info not in self._lines_info:
             err_msg = "Impossible to plot line info \"{}\" for line." \
                       " Possible values are {}"
-            raise PlotError(err_msg.format(line_info, str(self.lines_info)))
+            raise PlotError(err_msg.format(line_info, str(self._lines_info)))
         if load_info not in self._loads_info:
             err_msg = "Impossible to plot load info \"{}\" for line." \
                       " Possible values are {}"
-            raise PlotError(err_msg.format(load_info, str(self.loads_info)))
+            raise PlotError(err_msg.format(load_info, str(self._loads_info)))
         if gen_info not in self._gens_info:
             err_msg = "Impossible to plot gen info \"{}\" for line." \
                       " Possible values are {}"
-            raise PlotError(err_msg.format(gen_info, str(self.gens_info)))
+            raise PlotError(err_msg.format(gen_info, str(self._gens_info)))
 
-        line_unit = self._info_to_units[line_info]
+        line_values = None
+        line_unit = ""
+        if line_info is not None:
+            line_unit = self._info_to_units[line_info]
         if line_info == "rho":
-            line_values = observation.rho
+            line_values = observation.rho * 100.0
         if line_info == "p":
             line_values = observation.p_or
         if line_info == "a":
@@ -514,13 +552,19 @@ class BasePlot(ABC):
         if line_info == "v":
             line_values = observation.v_or
 
-        load_unit = self._info_to_units[load_info]
+        load_values = None
+        load_unit = ""
+        if load_info is not None:
+            load_unit = self._info_to_units[load_info]
         if load_info == "p":
             load_values = copy.copy(observation.load_p) * -1.0
         if load_info == "v":
             load_values = observation.load_v
 
-        gen_unit = self._info_to_units[gen_info]
+        gen_values = None
+        gen_unit = ""
+        if gen_info is not None:
+            gen_unit = self._info_to_units[gen_info]
         if gen_info == "p":
             gen_values = observation.prod_p
         if gen_info == "v":
@@ -579,23 +623,14 @@ class BasePlot(ABC):
         res: ``object``
             The figure updated with the data from the new observation.
         """
-
-        # Cope with non provided values
-        if line_values is None:
-            line_values = list(range(self.observation_space.n_line))
-        if load_values is None:
-            load_values = list(range(self.observation_space.n_load))
-        if gen_values is None:
-            gen_values = list(range(self.observation_space.n_gen))
-
         # Check values are in the correct format
-        if len(line_values) != self.observation_space.n_line:
+        if line_values is not None and len(line_values) != self.observation_space.n_line:
             raise PlotError("Impossible to display these values on the powerlines: there are {} values"
                             "provided for {} powerlines in the grid".format(len(line_values), self.observation_space.n_line))
-        if len(load_values) != self.observation_space.n_load:
+        if load_values is not None and len(load_values) != self.observation_space.n_load:
             raise PlotError("Impossible to display these values on the loads: there are {} values"
                             "provided for {} loads in the grid".format(len(load_values), self.observation_space.n_load))
-        if len(gen_values) != self.observation_space.n_gen:
+        if gen_values is not None and len(gen_values) != self.observation_space.n_gen:
             raise PlotError("Impossible to display these values on the generators: there are {} values"
                             "provided for {} generators in the grid".format(len(gen_info), self.observation_space.n_gen))
 
