@@ -6,18 +6,76 @@
 # SPDX-License-Identifier: MPL-2.0
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
 
+import io
 import numpy as np
-import matplotlib.pyplot as plt
+
 from matplotlib.path import Path
+
+from grid2op.PlotGrid.BasePlot import BasePlot
+from grid2op.PlotGrid.PlotUtil import PlotUtil as pltu
 import matplotlib.patches as patches
 from matplotlib.lines import Line2D
 
-from grid2op.PlotGrid.BasePlot import BasePlot
-from grid2op.PlotGrid.LayoutUtil import layout_obs_sub_load_and_gen
-from grid2op.PlotGrid.PlotUtil import PlotUtil as pltu
-
 
 class PlotMatplot(BasePlot):
+    """
+    Attributes
+    ----------
+
+    width: ``int``
+        Width of the figure in pixels
+    height: ``int``
+        Height of the figure in pixel
+    dpi: ``int``
+        Dots per inch, to convert pixels dimensions into inches
+    _scale: ``float``
+        Scale of the drawing in arbitrary units
+    _sub_radius: ``int``
+        Substation circle size 
+    _sub_face_color: ``str``
+        Substation circle fill color
+    _sub_edge_color: ``str``
+        Substation circle edge color
+    _sub_txt_color: ``str``
+        Substation info text color
+    _load_radius: ``int``
+        Load circle size
+    _load_face_color: ``str``
+        Load circle fill color
+    _load_edge_color: ``str``
+        Load circle edge color
+    _load_txt_color: ``str``
+        Load info text color
+    _load_line_color: ``str``
+        Color of the line from load to substation
+    _load_line_width: ``int``
+        Width of the line from load to substation
+    _gen_radius: ``int``
+        Generators circle size
+    _gen_face_color: ``str``
+        Generators circle fill color
+    _gen_edge_color: ``str``
+        Generators circle edge color
+    _gen_txt_color: ``str``
+        Generators info txt color
+    _gen_line_color: ``str``
+        Color of the line form generator to substation
+    _gen_line_width: ``str``
+        Width of the line from generator to substation
+    _line_color_scheme: ``list``
+        List of color strings to color powerlines based on rho values
+    _line_color_width: ``int``
+        Width of the powerlines lines
+    _line_bus_radius: ``int``
+        Size of the bus display circle
+    _line_bus_face_colors: ``list``
+        List of 3 colors strings, each corresponding to the fill color of the bus circle
+    _line_arrow_len: ``int``
+        Length of the arrow on the powerlines
+    _line_arrow_width: ``int``
+       Width of the arrow on the powerlines
+    """
+
     def __init__(self,
                  observation_space,
                  width=1280,
@@ -28,9 +86,8 @@ class PlotMatplot(BasePlot):
                  sub_radius = 15,
                  load_radius = 8,
                  gen_radius = 8):
-        self._scale = scale
         self.dpi = dpi
-        super().__init__(observation_space, width, height, grid_layout)
+        super().__init__(observation_space, width, height, scale, grid_layout)
 
         self._sub_radius = sub_radius
         self._sub_face_color = "w"
@@ -55,15 +112,15 @@ class PlotMatplot(BasePlot):
         #self._line_color_scheme = cm.get_cmap("inferno")(cx)
         self._line_color_scheme = [ "blue", "orange", "red"]
         self._line_color_width = 1
-        self._line_bus_radius = 5
-        self._line_bus_face_colors = ["black", "red", "magenta"]
+        self._line_bus_radius = 6
+        self._line_bus_face_colors = ["black", "red", "lime"]
         self._line_arrow_len = 10
         self._line_arrow_width = 10.0
 
         self.xlim = [0, 0]
-        self.xpad = 50
+        self.xpad = 20
         self.ylim = [0, 0]
-        self.ypad = 50
+        self.ypad = 20
 
     def _v_textpos_from_dir(self, dirx, diry):
         if diry > 0:
@@ -80,6 +137,8 @@ class PlotMatplot(BasePlot):
             return "right"
         
     def create_figure(self):
+        # lazy loading of graphics library (reduce loading time) [and mainly because matplolib has weird impact on argparse)
+        import matplotlib.pyplot as plt
         w_inch = self.width / self.dpi
         h_inch = self.height / self.dpi
         f = plt.figure(figsize=(w_inch, h_inch), dpi=self.dpi)
@@ -93,16 +152,14 @@ class PlotMatplot(BasePlot):
         self.ax = figure.subplots()
 
     def convert_figure_to_numpy_HWC(self, figure):
-        figure.canvas.draw()
         w, h = figure.canvas.get_width_height()
-        buf = np.fromstring(figure.canvas.tostring_rgb(), dtype=np.uint8)
-        buf = np.reshape(buf, (h, w, 3))
-        return buf
-
-    def compute_grid_layout(self, obs_space, grid_layout = None):
-        # We overload to specify the scale
-        # Also we expect obs_space has a grid_layout
-        return layout_obs_sub_load_and_gen(obs_space, self._scale, True)
+        buf = io.BytesIO()
+        figure.canvas.print_raw(buf)
+        buf.seek(0)
+        img_arr = np.frombuffer(buf.getvalue(), dtype=np.uint8)
+        buf.close()
+        img_arr = np.reshape(img_arr, (h, w, 4))
+        return img_arr
 
     def _draw_substation_txt(self, pos_x, pos_y, text):
         self.ax.text(pos_x, pos_y, text,
@@ -138,6 +195,7 @@ class PlotMatplot(BasePlot):
         self.ax.text(txt_x, txt_y, text,
                      color=self._load_txt_color,
                      horizontalalignment=ha,
+                     fontsize='small',
                      verticalalignment=va)
     
     def _draw_load_name(self, pos_x, pos_y, txt):
@@ -191,9 +249,10 @@ class PlotMatplot(BasePlot):
         self.ylim[1] = max(self.ylim[1], pos_y)
         self._draw_load_line(pos_x, pos_y, sub_x, sub_y)
         self._draw_load_circle(pos_x, pos_y)
-        load_txt = load_name + ":\n"
-        load_txt += pltu.format_value_unit(load_value, load_unit)
-        self._draw_load_txt(pos_x, pos_y, sub_x, sub_y, load_txt)
+        if load_value is not None:
+            load_txt = load_name + ":\n"
+            load_txt += pltu.format_value_unit(load_value, load_unit)
+            self._draw_load_txt(pos_x, pos_y, sub_x, sub_y, load_txt)
         self._draw_load_name(pos_x, pos_y, str(load_id))
         load_dir_x, load_dir_y = pltu.norm_from_points(sub_x, sub_y, pos_x, pos_y)
         self._draw_load_bus(sub_x, sub_y, load_dir_x, load_dir_y, load_bus)
@@ -215,6 +274,7 @@ class PlotMatplot(BasePlot):
         self.ax.text(txt_x, txt_y, text,
                      color=self._gen_txt_color,
                      wrap=True,
+                     fontsize='small',
                      horizontalalignment=ha,
                      verticalalignment=va)
 
@@ -269,10 +329,11 @@ class PlotMatplot(BasePlot):
         self.ylim[1] = max(self.ylim[1], pos_y)
         self._draw_gen_line(pos_x, pos_y, sub_x, sub_y)
         self._draw_gen_circle(pos_x, pos_y)
-        gen_txt = gen_name + ":\n"
-        gen_txt += pltu.format_value_unit(gen_value, gen_unit)
+        if gen_value is not None:
+            gen_txt = gen_name + ":\n"
+            gen_txt += pltu.format_value_unit(gen_value, gen_unit)
+            self._draw_gen_txt(pos_x, pos_y, sub_x, sub_y, gen_txt)
         self._draw_gen_name(pos_x, pos_y, str(gen_id))
-        self._draw_gen_txt(pos_x, pos_y, sub_x, sub_y, gen_txt)
         gen_dir_x, gen_dir_y = pltu.norm_from_points(sub_x, sub_y, pos_x, pos_y)
         self._draw_gen_bus(sub_x, sub_y, gen_dir_x, gen_dir_y, gen_bus)
 
@@ -289,12 +350,13 @@ class PlotMatplot(BasePlot):
                             text):
         pos_x, pos_y = pltu.middle_from_points(pos_or_x, pos_or_y, pos_ex_x, pos_ex_y)
         off_x, off_y = pltu.orth_norm_from_points(pos_or_x, pos_or_y, pos_ex_x, pos_ex_y)
-        txt_x = pos_x + off_x * self._load_radius
-        txt_y = pos_y + off_y * self._load_radius
+        txt_x = pos_x + off_x * (self._load_radius / 2)
+        txt_y = pos_y + off_y * (self._load_radius / 2)
         ha = self._h_textpos_from_dir(off_x, off_y)
         va = self._v_textpos_from_dir(off_x, off_y)
         self.ax.text(txt_x, txt_y, text,
                      color=self._gen_txt_color,
+                     fontsize='small',
                      horizontalalignment=ha,
                      verticalalignment=va)
     
@@ -356,16 +418,18 @@ class PlotMatplot(BasePlot):
                        or_bus, pos_or_x, pos_or_y,
                        ex_bus, pos_ex_x, pos_ex_y):
         rho = observation.rho[line_id]
-        color_idx = int(rho * len(self._line_color_scheme[:-1]))
-        color = self._line_color_scheme[color_idx] if connected else "black"
+        n_colors = len(self._line_color_scheme) - 1
+        color_idx = max(0, min(n_colors, int(rho * n_colors)))
+        color = self._line_color_scheme[color_idx] if connected and rho > 0.0 else "black"
         line_style = "-" if connected else "--"
         self._draw_powerline_line(pos_or_x, pos_or_y,
                                   pos_ex_x, pos_ex_y,
                                   color, line_style)
-        txt = pltu.format_value_unit(line_value, line_unit)
-        self._draw_powerline_txt(pos_or_x, pos_or_y,
-                                 pos_ex_x, pos_ex_y,
-                                 txt)
+        if line_value is not None:
+            txt = pltu.format_value_unit(line_value, line_unit)
+            self._draw_powerline_txt(pos_or_x, pos_or_y,
+                                     pos_ex_x, pos_ex_y,
+                                     txt)
 
         or_dir_x, or_dir_y = pltu.norm_from_points(pos_or_x, pos_or_y, pos_ex_x, pos_ex_y)
         self._draw_powerline_bus(pos_or_x, pos_or_y,
@@ -416,3 +480,4 @@ class PlotMatplot(BasePlot):
     def plot_postprocess(self, figure, observation, update):
         self.ax.set_xlim(self.xlim[0] - self.xpad, self.xlim[1] + self.xpad)
         self.ax.set_ylim(self.ylim[0] - self.ypad, self.ylim[1] + self.ypad)
+        figure.tight_layout()
